triton.language.extra.cann.libdevice.llrint#
- triton.language.extra.cann.libdevice.llrint(arg0: None, _semantic: None = 'None')#
Rounds a floating-point number to the nearest 64-bit integer value.
- 参数:
arg0 (scalar or tl.tensor) --
x. Supported dtype(s):float32.- 返回:
The rounded 64-bit integer.
- 返回类型:
int64
示例
import os os.environ.setdefault("TRITON_ENABLE_LIBDEVICE_SIMT", "1") import pytest import triton import triton.language as tl import triton.language.extra.cann.libdevice as libdevice import torch from triton.backends.ascend.utils import triton_enable_libdevice_simt _SIMT_SKIP_MSG = ("SIMT libdevice ops require an Ascend 950 target " "with TRITON_ENABLE_LIBDEVICE_SIMT=1; skipping.") def torch_llrint_reference(x0): assert x0.device.type == "cpu" assert x0.dtype == torch.float32 return torch.round(x0).to(torch.int64) @triton.jit def triton_kernel(input0, output, n_elements, XBLOCK: tl.constexpr, XBLOCK_SUB: tl.constexpr): offset = tl.program_id(0) * XBLOCK base = tl.arange(0, XBLOCK_SUB) loops: tl.constexpr = XBLOCK // XBLOCK_SUB for loop in range(loops): x0 = offset + (loop * XBLOCK_SUB) + base mask = x0 < n_elements tmp0 = tl.load(input0 + (x0), mask=mask) tmp1 = libdevice.llrint(tmp0) tl.store(output + (x0), tmp1, mask=mask) @pytest.mark.skipif(not triton_enable_libdevice_simt(), reason=_SIMT_SKIP_MSG) def test_llrint(): x0 = (torch.rand((8, )) * 3.0 - 1.5).to(torch.float32) expected = (torch_llrint_reference(x0)).npu() x0 = x0.npu() output = torch.empty(8, dtype=torch.int64, device='npu') triton_kernel[(1, )](x0, output, 8, XBLOCK=8, XBLOCK_SUB=8, compile_mode='simt_only') output = output.cpu() expected = expected.cpu() torch.testing.assert_close(output, expected, rtol=0, atol=0) if __name__ == "__main__": if not triton_enable_libdevice_simt(): print(_SIMT_SKIP_MSG) else: test_llrint()
特殊说明
Platform and compilation modes:
Ascend 950 supports SIMD, SIMT.